@@ -2,15 +2,24 @@ package Provider::ConfigExtended;
 
 use Carp qw(croak);
 use DateTime;
+use Dancer2::Core::Types qw(Int);
 use List::Util qw(first);
 use Moo;
 extends "Dancer2::Plugin::Auth::Extensible::Provider::Config";
 use namespace::clean;
 
+has password_expiry_days => (
+    is  => 'ro',
+    isa => Int,
+);
+
 sub create_user {
-    my ( $self, %user ) = @_;
+    my $self = shift;
+    my %user = @_ == 1 && ref($_[0]) eq 'HASH' ? %{ $_[0] } : @_;
 
     my $username = delete $user{username};
+    croak "Username not supplied in args"
+      unless defined $username && $username ne '';
     croak "User already exists"
       if first { $_->{user} eq $username } @{ $self->users };
 
@@ -21,13 +30,18 @@ sub create_user {
 
 sub get_user_by_code {
     my ( $self, $code ) = @_;
-    my $user = first { $_->{pw_reset_code} eq $code } @{ $self->users };
+    croak "code needs to be specified"
+      unless $code && $code ne '';
+    my $user = first { $_->{pw_reset_code} && $_->{pw_reset_code} eq $code }
+      @{ $self->users };
     return unless $user;
     return $user->{user};
 }
 
 sub set_user_details {
     my ( $self, $username, %update ) = @_;
+    croak "Username to update needs to be specified"
+      unless $username && $username ne '';
     my $user = first { $_->{user} eq $username } @{ $self->users };
     return unless $user;
     foreach my $key ( keys %update ) {
@@ -38,14 +52,26 @@ sub set_user_details {
 
 sub set_user_password {
     my ( $self, $username, $password ) = @_;
+
+    croak "username and password must be defined"
+      unless defined $username && defined $password;
+
     my $encrypted = $self->encrypt_password($password);
-    $self->set_user_details( $username, pass => $encrypted );
+    $self->set_user_details(
+        $username,
+        pass      => $encrypted,
+        pw_changed => DateTime->now
+    );
 }
 
 sub password_expired {
     my ( $self, $user ) = @_;
+
+    croak "user must be specified"
+      unless defined $user && ref($user) eq 'HASH' && defined $user->{user};
+
     my $expiry = $self->password_expiry_days or return 0;
-    my $last_changed = $user->{pwchanged};
+    my $last_changed = $user->{pw_changed};
     return 1 unless $last_changed;
 
     my $duration = $last_changed->delta_days( DateTime->now );
